#ifndef _COMPORT_H_
#define _COMPORT_H_

#include <go32.h>
#include <dpmi.h>

const int COM_PORT_BUFFER_SIZE = 4096;
const int COM_PORT_BUFFER_FLOW_ON = 256;
const int COM_PORT_BUFFER_FLOW_OFF = 3072;
// define COM_PORT_XON_XOFF_ENABLED

struct SRegs8250
{
  int DLL;  // divisor latch low register (if LCR bit7 = 1) 
  int DLH;  // divisor latch high register (if LCR bit7 = 1) 
  int THR;  // transmit holding register 
  int RBR;  // receive holding register 
  int IER;  // interrupt enable register 
  int LCR;  // line control register 
  int MCR;  // modem control register 
  int LSR;  // line status register 
  int MSR;  // modem status register 
};

class CComPort
{
public:
  CComPort(int nComPort = 1, int BasePort = 0, int IRQ = 0, int Interrupt = 0);
  ~CComPort();

  void Create(int BasePort, int IRQ, int Interrupt);
  
  int ReadData(char *c);
  int WriteData(const char *c);

  int BufferLength()
  { return (m_BufferHead > m_BufferTail ? m_BufferHead - m_BufferTail : m_BufferTail - m_BufferHead); }

  void Flush()
  { m_BufferHead = m_BufferTail = 0; }
  
  int Port()
  { return m_nComPort; }

  friend void ComPortHandler();
  
  char  m_Buffer[COM_PORT_BUFFER_SIZE];
  int   m_BufferHead, m_BufferTail;
  int   m_nComPort;
  int   m_nIRQ;
  int   m_nInterrupt, m_nBasePort;
  int   m_bFlowSend, m_bFlowRecv;
  
  _go32_dpmi_seginfo    m_OldInfo, m_NewInfo;
  SRegs8250             m_Regs8250;
protected:
};

#endif
