#include <stdio.h>
#include <iostream.h>
#include <conio.h>
#include <sys/stat.h>
#include <time.h>
#include <string.h>
#include <dos.h>
#include "packet.h"
#include "comport.h"
#include "do.h"

extern CComPort * pComPort;
extern int bDebug;

int XmodemGet(const char * Filename)
{
  LogMessage(LOG_EVENT, "Receiving file");
  LogMessage(LOG_MESSAGE, Filename);
  ResetEscape();
  
  FILE * OutputFile = fopen(Filename, "wb"); 
  if (!OutputFile)  
  {
    cout << "Error: File could not be opened" << endl;
    return 0;
  }
  
  char c = 0;
  
  int FileSize = 0;
  int bDone = 0;
  c = 21;
  pComPort->Flush();
  pComPort->WriteData(&c);
  int Packet = 1;
  while (!bDone)
  {
    if (!GetChar(&c))
    {
      c = 24;
      pComPort->WriteData(&c);
      pComPort->WriteData(&c);
      pComPort->WriteData(&c);
      cout << "Transfer cancelled locally" << endl;
      fclose(OutputFile);
      return 0;
    }

    switch (c)
    {
    case 1: // SOH
      {  
        cout << "Buffer: " 
             << pComPort->BufferLength() << " " << pComPort->m_BufferHead
             << " " << pComPort->m_BufferTail << endl;

        if (bDebug)
          cout << "Got SOH" << endl;
        else
        {
          gotoxy(1, wherey());
          clreol();
          cout << "Packet " << Packet << flush;
        }
        unsigned int PacketCount, PacketCheck, PacketChecksum, Checksum = 0;
        GetChar(&c);
        PacketCount = (unsigned char)c;
        GetChar(&c);
        PacketCheck = (unsigned char)c;
        char Data[128];
        for (int i = 0; i < 128; i++)
        {
          GetChar(&Data[i]);
          Checksum = (Checksum + (unsigned char)Data[i]) % 256;
        }
        GetChar(&c);
        PacketChecksum = (unsigned char)c;
        if (bDebug)
          cout << "Got Packet #" << PacketCount << " (" << Packet << ", " 
               << PacketCheck << "), Checksum = " << PacketChecksum << " (" 
               << Checksum << ")" << endl;
        if (!HitEscape())
        {
          if (Checksum != PacketChecksum)
          {
            if (bDebug)
              cout << "Bad packet, sending NAK and flushing buffer" << endl;
            delay(100);
            pComPort->Flush();
            c = 21;
            pComPort->WriteData(&c);
          }
          else
          {  
            if (bDebug)
              cout << "Good packet, sending ACK" << endl;
            c = 6;
            pComPort->WriteData(&c);
            fwrite(Data, 1, 128, OutputFile);
            fflush(OutputFile);
            FileSize += 128;
            Packet++;
            if (Packet == 256)
              Packet = 1;
          }
        }
      }
      break;
    case 4: // EOT, done
      {
        if (bDebug)
          cout << "Got EOT, sending ACK" << endl;
        c = 6;
        pComPort->WriteData(&c);
        bDone = 1;
      }
      break;
    case 21:
      {
        if (bDebug)
          cout << "Got CAN, aborting" << endl;
        bDone = 1;
      }
      break;
    default:
      {
        if (bDebug)
          cout << "Got invalid character: " << (int)c << ", sending NAK and flushing buffer" << endl;
        delay(100);
        pComPort->Flush();
        c = 21;
        pComPort->WriteData(&c);
      }
    }
  }
  fclose(OutputFile);
  
  gotoxy(1, wherey());
  clreol();
  cout << "File received successfully (" << FileSize << " bytes)" << endl;
  
  return 1;
}

int XmodemPut(const char * Filename)
{
  LogMessage(LOG_EVENT, "Sending file");
  LogMessage(LOG_MESSAGE, Filename);
  ResetEscape();

  unsigned int StartTime = rawclock();

  FILE * OutputFile = fopen(Filename, "rb"); 
  if (!OutputFile)  
  {
    cout << "Error: File could not be opened" << endl;
    return 0;
  }
  
  struct stat FileInfo;
  fstat(fileno(OutputFile), &FileInfo);
  
  char c = 0;
  
  pComPort->Flush();
  do
  {
    if (!GetChar(&c))
    {
      c = 24;
      pComPort->WriteData(&c);
      pComPort->WriteData(&c);
      pComPort->WriteData(&c);
      cout << "Transfer cancelled locally" << endl;
      return 0;
    }
    if (bDebug)
      cout << "Received char: " << (int)c << endl;
  }
  while (c != 21 && c != 24);  // NAK and CAN respectively
  
  if (c == 24)
  {
    cout << "Transfer cancelled remotely" << endl;
    return 0;
  }
  
  int PacketCount = 1;
  int FileOffset = 0;
  int bDone = 0;

  while (!bDone)
  {
    pComPort->Flush();
    int bOkay = 0;
    char Data[128];
    memset(Data, 0, 128);
    int Size = fread(&Data, 1, 128, OutputFile);
    FileOffset += 128;
    
    // Check for EOF
    if (Size != 128)
      bDone = 1;
      
    // No need to send this packet, there won't be any data
    if (Size == 0) 
      bOkay = 1;

    while (!bOkay)
    {
      gotoxy(1, wherey());
      clreol();
      cout << "Packet " << PacketCount << " (" << int(FileOffset * 100 / FileInfo.st_size) << "%";
      if (FileOffset > 1000)
      {
        int CurrentTime = rawclock();
        double FinishRaw = (double)(CurrentTime - StartTime) * FileInfo.st_size / FileOffset - (CurrentTime - StartTime);
        int FinishTime = int(FinishRaw / 18.2);
        cout << ", " << FinishTime << " secs remain";
      }
      cout << ")" << flush;
      int Checksum = 0;
      // SOH
      c = 1;
      pComPort->WriteData(&c);

      // Packet count and check
      c = PacketCount;
      pComPort->WriteData(&c);
      c = 255 - c;
      pComPort->WriteData(&c);
  
      // Packet data
      for (int i = 0; i < 128; i++)
      {
        pComPort->WriteData(&Data[i]);
        Checksum = (Checksum + Data[i]) % 256;
      }    
  
      // Checksum
      c = Checksum;
      pComPort->WriteData(&c);
  
      // Get ACK or CAN
      int Status = GetChar(&c);
      if (bDebug && !HitEscape())
        cout << "Received char: " << (int)c << endl;
      if (!Status || c == 24) // CAN
      {
        c = 24;
        pComPort->WriteData(&c);
        pComPort->WriteData(&c);
        pComPort->WriteData(&c);
        cout << "Transfer cancelled" << endl;
        return 0;
      }

      if (c == 6) // ACK
      {
        bOkay = 1;
        PacketCount++;
        if (PacketCount == 256)
          PacketCount = 1;
      }
    }
  }
  
  // Send EOT (done)
  c = 4;
  pComPort->WriteData(&c);
  
  fclose(OutputFile);
  
  gotoxy(1, wherey());
  clreol();
  cout << "File sent successfully" << endl;
  
  return 1;
}
